#ifndef MAMMUTDEBUG_H
#define MAMMUTDEBUG_H

#include <iostream>

class NoDebug
{
public:
	typedef std::vector<int> StateStack;

	static void Message(char const* msg) { }
	static void EnterState(int state) { }
	static void WriteNextToken(int state, char const* name) { }
	static void ShiftToken(int state, char const* name) { }
	static void ReduceMessage(int yyn, int yyrline, short const* yyprhs, short const* yyrhs,
		char const* const* yytname, short const* yyr1) { }
	static void PrintStack(StateStack const& stateStack) { }
	static void DiscardToken(int yychar, char const* name) { }
	static void Error(char const* msg) { }
};

class ConsoleDebug
{
public:
	typedef std::vector<int> StateStack;

	static void Message(char const* msg)
	{
		std::cerr << msg << std::endl;
	}

	static void EnterState(int state)
	{
		std::cerr << "Entering state " << state << std::endl;
	}

	static void WriteNextToken(int state, char const* name)
	{
		std::cerr << "Next token is " << state << '(' << name << ')' << std::endl;
		/* Give the individual parser a way to print the precise meaning
		of a token, for further debugging info.  */
		//YYPRINT (stderr, yychar, yylval);
	}

	static void ShiftToken(int state, char const* name)
	{
		std::cerr << "Shifting token " << state << '(' << name << ')' << std::endl;
	}

	static void ReduceMessage(int yyn, int yyrline, short const* yyprhs, short const* yyrhs,
		char const* const* yytname, short const* yyr1)
	{
		std::cerr << "Reducing via rule " << yyn << " (line " << yyrline << ')' << std::endl;

		// Print the symbols being reduced, and their result.
		for(int i = yyprhs[yyn]; yyrhs[i] > 0; i++)
			std::cerr << yytname[yyrhs[i]] << ' ';
		std::cerr << "-> " << yytname[yyr1[yyn]] << std::endl;
	}

	static void PrintStack(StateStack const& stateStack)
	{
		std::cerr << "State stack now";
		for(StateStack::const_iterator iter = stateStack.begin(); iter != stateStack.end(); iter++)
			std::cerr << ' ' << *iter;
		std::cerr << std::endl;
	}

	static void DiscardToken(int yychar, char const* name)
	{
		std::cerr << "Discarding token " << yychar << " (" << name << ')' << std::endl;
	}

	static void Error(char const* msg)
	{
		std::cerr << "Error encountered: \"" << msg << '"' << std::endl;
	}
};

#endif //#MAMMUTDEBUG_H
